// JVMStarter.cpp : R\[ AvP[ṼGg |Cg`܂B
//

#include "stdafx.h"

#include <assert.h>
#include <process.h>

#include <exception>
#include <fstream>

#include "ConfigParser.hpp"
#include "JavaVMMainThread.hpp"
#include "ExceptionFilter.hpp"
#include "EventLog.hpp"

#include "JVMStarter.hpp"

unsigned __stdcall terminaterThreadProc( void* v_pParam )
{
	UnhandledExceptionHandler::setUnhandledExceptionHandler();

	JavaVMMainThread* pJVMThread = (JavaVMMainThread*) v_pParam;
	assert( pJVMThread != NULL );

	Sleep( 10 * 1000 );
	pJVMThread->stopInterrupt();

	return 0;
}

JVMStarter::JVMStarter( JVMEventListener& v_eventListener, const std::string& v_preloadConfPath ) throw ()
	: eventListener_( v_eventListener )
	, preloadConfPath_( v_preloadConfPath )
{
}

bool JVMStarter::preloadLibrary() throw ()
{
	if( preloadConfPath_.empty() ) {
		return true;
	}

	try{
		std::ifstream fin;
		fin.open( preloadConfPath_.c_str() );
		if( ! fin.is_open() ) {
			return true;
		}
		
		std::string line;
		while( getline( fin, line ) ) {
			CHAR dllPath[ MAX_PATH ];
			ExpandEnvironmentStrings( line.c_str(), dllPath, MAX_PATH );
			LPCSTR p = dllPath;
			while(*p) {
				if( *p == '#' || *p == ';' ) {
					break;
				}
				if( ! ( *p > 0 && *p <= ' ' ) ) {
					if( LoadLibrary( p ) == NULL ) {
						throw std::runtime_error( dllPath );
					}
					break;
				}
				p = CharNext( p );
			}
		}
		fin.close();
		return true;
	}
	catch( const std::exception& v_exception ) {
		eventListener_.notifyLibraryLoadFailure( v_exception.what() );
	}
	catch( ... ) {
		eventListener_.notifyLibraryLoadFailure( "sȗO܂B" );
	}
	return false;
}

void JVMStarter::execute( const std::string& v_eventSourceName, const std::string& v_configPath ) throw()
{
	try{
		ConfigParser parser;
		parser.parse( v_configPath );

		JavaVMMainThread jvm( eventListener_, parser.getJavaVMStartupInfo() );
		jvm.start();
		eventListener_.notifyJVMStarted();

		unsigned tid = 0;
		HANDLE hThread = (HANDLE) _beginthreadex( NULL, 0, terminaterThreadProc, &jvm, 0, &tid );
		CloseHandle( hThread );

		const ConfigParser::size_type mx = parser.size();
		ConfigParser::size_type idx = 1;
		for( ConfigParser::const_iterator p=parser.begin(), last=parser.end();
			p != last;
			++p, ++idx)
		{
			const StartupInfo& startupInfo = *p;
			if( ! jvm.invokeClassMethod( startupInfo, ( idx == mx ) || ! startupInfo.isFork() ) ) {
				break;
			}
		}
		jvm.waitForFinishedAll();
		eventListener_.notifyJVMExited( 0 );
	}
	catch( const std::exception& v_exception ) {
		eventListener_.notifyException( v_exception.what() );
	}
	catch( ... ) {
		eventListener_.notifyException( "JVMStarter#executeŕsȗO܂B" );
	}
}
